// 16 Colors BMP to SWAN Tiles Gfx
// by Orion_ [2008]

#define _CRT_SECURE_NO_DEPRECATE

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

unsigned char tilebank[32*1024];	// max tile bank for wonderswan color

unsigned short	CheckTiles(int ntile)
{
	int	i, j;
	unsigned char *base, *comp;
	unsigned char hflip[4*8], vflip[4*8], hvflip[4*8];

	if (ntile > 0)
	{
		base = tilebank;
		comp = &tilebank[ntile*4*8];

		for (j = 0; j < 8; j++)
			for (i = 0; i < 4; i++)
				hflip[(j*4)+i] = (comp[(j*4)+(3-i)] >> 4) | (comp[(j*4)+(3-i)] << 4);

		for (j = 0; j < 8; j++)
			for (i = 0; i < 4; i++)
				vflip[(j*4)+i] = comp[((7-j)*4)+i];

		for (j = 0; j < 8; j++)
			for (i = 0; i < 4; i++)
				hvflip[(j*4)+i] = (comp[((7-j)*4)+(3-i)] >> 4) | (comp[((7-j)*4)+(3-i)] << 4);

		for (i = 0; i < ntile; i++)
		{
			if (memcmp(base, comp, 4*8) == 0)
				return (i);
			if (memcmp(base, hflip, 4*8) == 0)
				return (i | 0x4000);
			if (memcmp(base, vflip, 4*8) == 0)
				return (i | 0x8000);
			if (memcmp(base, hvflip, 4*8) == 0)
				return (i | 0x8000 | 0x4000);
			base += 4*8;
		}
	}
	return (0xFFFF);
}

int					main(int argc, char *argv[])
{
	FILE			*bmpfile, *gfxfile, *palfile, *mapfile;
	char			gfxname[255], palname[255], mapname[255];
	unsigned short	xsize, ysize, i, j, k, l, color, rtile;
	unsigned char	bpp, r, g, b, o, bt;
	unsigned int	size, map = 0, ntile = 0, mtile = 0, stile = 0, mono = 0;

	printf("bmp2swan v0.4 by Orion_ [2004-2008] http://onorisoft.free.fr/\n\n");

	if (!(argc > 1))
	{
		printf("Usage: bmp2swan bmpfile <-map|-4col>\n"\
			   " <none> : Generate 16 colors chunky gfx data (for wonderswan color)\n"\
			   "  -map  : Generate an optimized Map by removing same tiles (h/vflip supported)\n"\
			   "  -4col : Generate 4 colors planar gfx data (for wonderswan mono)\n\n");
		system("PAUSE");
		return (0);
	}

	bmpfile = fopen(argv[1], "rb");
	if (!bmpfile)
	{
		printf("Can't open '%s'\n", argv[1]);
		return (-1);
	}

	strcpy(gfxname, argv[1]);	strchr(gfxname, '.')[1] = '\0';	strcat(gfxname, "gfx");
	strcpy(palname, argv[1]);	strchr(palname, '.')[1] = '\0';	strcat(palname, "pal");
	strcpy(mapname, argv[1]);	strchr(mapname, '.')[1] = '\0';	strcat(mapname, "map");

	fseek(bmpfile, 0, SEEK_END);
	size = ftell(bmpfile);

	fseek(bmpfile, 18, SEEK_SET);
	fread(&xsize, 1, 2, bmpfile);
	fseek(bmpfile, 22, SEEK_SET);
	fread(&ysize, 1, 2, bmpfile);
	fseek(bmpfile, 28, SEEK_SET);
	fread(&bpp, 1, 1, bmpfile);

	if (bpp != 4)
	{
		fclose(bmpfile);
		printf("BMP must be in 16 colors !\n");
		return (-1);
	}

	if ((xsize & 7) || (ysize & 7))
	{
		fclose(bmpfile);
		printf("BMP X and Y size must be a multiple of 8 !\n");
		return (-1);
	}

	gfxfile = fopen(gfxname, "wb");
	if (!gfxfile)
	{
		fclose(bmpfile);
		printf("can't create '%s'\n", gfxname);
		return (-1);
	}

	if (argc > 2)
	{
		if (!strcmp(argv[2], "-4col"))
		{
			mono = 1;
		}
	}

	if (mono == 0)
	{
		palfile = fopen(palname, "wb");
		if (!palfile)
		{
			fclose(bmpfile);
			fclose(gfxfile);
			printf("can't create '%s'\n", palname);
			return (-1);
		}
	}

	if (argc > 2)
	{
		if (!strcmp(argv[2], "-map"))
		{
			map = 1;
			mapfile = fopen(mapname, "wb");
			if (!mapfile)
			{
				fclose(bmpfile);
				fclose(gfxfile);
				fclose(palfile);
				printf("can't create '%s'\n", mapname);
				return (-1);
			}
		}
	}

	printf("Xsize: %u\nYsize: %u\n\n", xsize, ysize);
	xsize /= 2;	// 2 pixels per byte

	if (mono == 0)
	{
		fseek(bmpfile, 54, SEEK_SET);
		printf("Converting bmp palette...\n");
		for (i = 0; i < 16; i++)
		{
			fread(&b, 1, 1, bmpfile);
			fread(&g, 1, 1, bmpfile);
			fread(&r, 1, 1, bmpfile);
			fread(&o, 1, 1, bmpfile);
			color = ((r >> 4) << 8) + ((g >> 4) << 4) + (b >> 4);	// swan color format 4:4:4
			fwrite(&color, 1, 2, palfile);
		}
		fclose(palfile);
	}
	else
	{
		printf("Creating tiles from BMP data... Wonderswan Mono mode !\n");
		for (i = 0; i < ysize; i += 8)
		{
			for (j = 0; j < xsize; j += 4)
			{
				for (r = 0, b = 0, l = 0; l < 8; l++)
				{
					fseek(bmpfile, size - ((xsize * i) + (xsize * l) + (xsize - j)), SEEK_SET);

					for (k = 0; k < 4; k++)
					{
						fread(&bt, 1, 1, bmpfile);
						r <<= 1; if ((bt >> 4) & 1) r |= 1;	// planar
						r <<= 1; if ( bt  & 1)      r |= 1;
						b <<= 1; if ((bt >> 4) & 2) b |= 1;
						b <<= 1; if ( bt  & 2)      b |= 1;
					}
					fwrite(&r, 1, 1, gfxfile);
					fwrite(&b, 1, 1, gfxfile);
				}
			}
		}
		goto endo;
	}

	printf("Creating tiles from BMP data... Wonderswan Color Mode !\n");
	for (i = 0; i < ysize; i += 8)
	{
		for (j = 0; j < xsize; j += 4)
		{
			for (l = 0; l < 8; l++)
			{
				fseek(bmpfile, size - ((xsize * i) + (xsize * l) + (xsize - j)), SEEK_SET);
				for (k = 0; k < 4; k++)
				{
					fread(&bt, 1, 1, bmpfile);
					tilebank[mtile++] = bt;
				}
			}

			rtile = CheckTiles(ntile);
			if ((map == 0) || (rtile == 0xFFFF))
			{
				ntile++;
				fwrite(&tilebank[mtile-(4*8)], 1, 4*8, gfxfile);
			}
			else
			{
				mtile -= 4*8;
			}

			if (map)
			{
				if (rtile != 0xFFFF)
					stile++;
				else
					rtile = ntile - 1;
				fwrite(&rtile, 2, 1, mapfile);
			}
		}
	}

endo:
	fclose(bmpfile);
	fclose(gfxfile);
	if (map)
		printf("Map created: %d tiles (%d bytes) removed\n", stile, stile*4*8);
	printf("done...\n");

	return (0);
}
