#!/usr/bin/env python

import numpy as np
import sys

rom_sizes = [128, 256, 512, 1024, 2048, 3072, 4096, 6144, 8192, 16384]

SEGMENT = 0x4000
ROMSIZE = 3

if len(sys.argv) < 3:
    print("FinalizeWSCRom.py <in.com> <out.wsc> [input origin point in hex, default=0, 0x100 is a possibility]")
    exit()

fin = open(sys.argv[1], 'rb')
ain = np.fromfile(fin, dtype=np.uint8)

org = 0
if len(sys.argv) >= 4:
    org = int(sys.argv[3], 16)

a = np.zeros(shape=(1024*rom_sizes[ROMSIZE],), dtype=np.uint8)  
a[org:org+len(ain)] = ain

rom_metadata_type = np.dtype([
    ('0xea', 'u1'),
    ('org', '<u2'),
    ('segment', '<u2'),
    ('zero', 'u1'),
    ('publisher_id', 'u1'),
    ('system', 'u1'),
    ('game_id', 'u1'),
    ('revision', 'u1'),
    ('rom_size', 'u1'),
    ('save_type', 'u1'),
    ('flags', 'u1'),
    ('RTC_present', 'u1'),
    ('checksum', '<u2')])

#metadata = np.array([(0xea, org, SEGMENT, 0, 1, 1, 0xb0, 0xde, ROMSIZE, 0, 4, 0, 0xcccc)], dtype=rom_metadata_type)
metadata = np.array([(0xea, org, SEGMENT, 0, 0xA, ord('X'), ord('L'), ord('M'), ROMSIZE, 1, 4, 0, 0xcccc)], dtype=rom_metadata_type)
mbytes = metadata.view('u1')

print(metadata)
print(mbytes)

a[-len(mbytes):] = mbytes

try:
    a16 = a.view(np.uint16)
    a16[-1] = int(a16[:-1].sum()) & 0xffff # checksum?
    print(hex(a16[-1]))
except TypeError as e:
    print("Warning, could not calculate checksum due to numpy mysteries.")
    print("Here's an exception trace to ignore:")
    print(e)


fout = open(sys.argv[2], "wb")
a.tofile(fout)
fout.close()
print("rom image OK")
