#!/usr/bin/env python3

import os
from sys import argv
import glob

gfxs = glob.glob(argv[1] + "/*.gfx")
gfxs = sorted(gfxs)
gfxs = [fn.replace("\\", "/") for fn in gfxs]

names = [g.replace(argv[1]+"/","")[:-4] for g in gfxs]
names_ = [n.replace("-", "_") for n in names]

fasm = open(argv[1] + '.inc', 'tw')
fc = open(argv[1] + '.h', 'tw')

def toinc(s):
    fasm.write(s)
    fasm.write("\n")

def toh(s):
    fc.write(s)
    fc.write("\n")

toh(f'#ifndef ___{argv[1]}_H')
toh(f'#define ___{argv[1]}_H')
toh(f'#include<stdint.h>')

for i in range(len(names_)):
    n = names_[i]
    g = gfxs[i]
    m = g.replace(".gfx",".map")
    p = g.replace(".gfx",".pal")
    toinc(f'SECTION _DATA_{n} CLASS=FAR_DATA')
    toinc(f'db "/{n} "')
    toinc(f'global _{n}_start')
    toinc(f'_{n}_start: incbin "{g}"')
    toinc(f'_{n}_end:')
    toh(f'extern uint16_t far {n}_start[];')
    toinc(f'global _{n}_map_start')
    if os.path.isfile(m):
        toinc(f'_{n}_map_start: incbin "{m}"')
    else:
        toinc(f'_{n}_map_start: resb 2048 ')
    toinc(f'_{n}_map_end:')
    toh(f'extern uint16_t far {n}_map_start[];')
    if os.path.isfile(p):
        toinc(f'global _{n}_pal_start')
        toinc(f'_{n}_pal_start: incbin "{p}"')
    else:
        toinc(f'global _{n}_pal_start')
        toinc(f'_{n}_pal_start: resb 32')
    toinc(f'_{n}_pal_end:')
    toh(f'extern uint16_t far {n}_pal_start[];')
    toinc("")


toinc(f"SECTION _TEXT")
toinc(f"db 'image pointer tables '")
toinc(f"global _number_of_images_in_{argv[1]}")
toinc(f"_number_of_images_in_{argv[1]}: dw {len(names)}")
toinc(f"global _GFXS_{argv[1]}")
toinc(f"_GFXS_{argv[1]}:")
for n in names_:
    toinc(f"dw _{n}_start")
    toinc(f"dw seg _{n}_start")

toinc("")
toinc("")

toinc(f"global _PALS_{argv[1]}")
toinc(f"_PALS_{argv[1]}:")
for n in names_:
    toinc(f"dw _{n}_pal_start")
    toinc(f"dw seg _{n}_pal_start")

toinc("")
toinc("")

toinc(f"global _MAPS_{argv[1]}")
toinc(f"_MAPS_{argv[1]}:")
for n in names_:
    toinc(f"dw _{n}_map_start")
    toinc(f"dw seg _{n}_map_start")

toh(f'extern void far * far *GFXS_{argv[1]};')
toh(f'extern void far * far *MAPS_{argv[1]};')
toh(f'extern uint16_t far * far *PALS_{argv[1]};')

toh("#endif")
